@@ -171,11 +171,19 @@ sub get_sidechain_name
         my $element = $elements{$symbol}->{prefix};
         my $name = ChemOnomatopist::Name->new;
 
+        if( $chain->nonstandard_valences ) {
+            $name->append_locants( map { 'λ' . $_ } $chain->nonstandard_valences );
+        }
+
         if(      $symbol eq 'Al' ) {
             $element = 'aluman';
         } elsif( $symbol eq 'As' ) {
             $element = 'arsan';
-        } elsif( $symbol !~ /^(B|P)$/ ) {
+        } elsif( $symbol eq 'P' ) {
+            $element = 'phosphan';
+        } elsif( $symbol eq 'S' ) {
+            $element = 'sulfan';
+        } elsif( $symbol ne 'B' ) {
             $element =~ s/a$//;
         }
         if( $symbol =~ /^(Cl|Br|F|I)$/ ) {
@@ -222,7 +230,9 @@ sub get_sidechain_name
 
     # Collecting names of all the attachments
     my $name = ChemOnomatopist::Name->new;
-    for my $attachment_name (sort { $a cmp $b } keys %attachments) {
+    my @order = sort { $a cmp $b } keys %attachments;
+    for my $i (0..$#order) {
+        my $attachment_name = $order[$i];
         my( $attachment, @positions ) = @{$attachments{$attachment_name}};
 
         if( $chain->needs_substituent_locants ) {
@@ -249,6 +259,12 @@ sub get_sidechain_name
             }
         }
 
+        # Enclose all but the first attachment in a chain not needing substituent locants
+        if( !$attachment->is_enclosed && !$chain->needs_substituent_locants &&
+            @order > 1 && $i > 0 ) {
+            $attachment->bracket;
+        }
+
         $name .= $attachment;
     }
 
@@ -302,8 +318,14 @@ sub get_sidechain_name
                 $name->pop_yl;
             }
         }
-        $name .= $chain->prefix;
-        $name->pop_yl;
+
+        my $prefix = $chain->prefix;
+        if( $prefix eq 'nyl' && $name->ends_with_element && $name->[-1] eq 'sila' ) {
+            $name->[-1]{value} =~ s/a$//;
+        } else {
+            $name .= $prefix;
+            $name->pop_yl;
+        }
 
         if( $branches_at_start > 1 ) {
             my $branch_point = first { $chain[$_] == $start } 0..$#chain;
@@ -378,6 +400,15 @@ sub get_mainchain_name
                 push @senior_group_attachments, $i;
             } else {
                 my $attachment = get_sidechain_name( $graph, $atom, $group ? $group : $neighbour );
+                if( $chain->needs_ane_suffix ) {
+                    if( $attachment eq 'phenyl' ) {
+                        $attachment = ChemOnomatopist::Name->new( 'benzene' );
+                    } elsif( $attachment =~ /yl$/ ) {
+                        # FIXME: Properly detect the 'yl' suffix and replace it with suffix object
+                        $attachment->pop_yl;
+                        $attachment .= $attachment eq 'hydrazin' ? 'e' : 'ane';
+                    }
+                }
                 $attachments{$attachment} = [ $attachment ] unless $attachments{$attachment};
                 push @{$attachments{$attachment}}, $i;
             }
@@ -413,6 +444,7 @@ sub get_mainchain_name
                 ( $attachment =~ /^dec/ || # BBv2 P-16.3.4 (d)
                   $attachment =~ /^sulfanyl/ || # BBv3 P-16.3.6 (b)
                  !$attachment->is_simple ||
+                  $attachment->has_isotope ||
                   $attachment->has_substituent_locant ) ) {
                 $attachment->bracket;
             }
@@ -433,6 +465,12 @@ sub get_mainchain_name
               $attachment->bracket;
         }
 
+        if( !$attachment->is_enclosed &&
+            !$chain->needs_substituent_locants &&
+             $i > 0 ) {
+            $attachment->bracket;
+        }
+
         if( $chain->isa( ChemOnomatopist::Group::Amidine:: ) &&
             !$attachment->is_enclosed && $i == $#order ) {
             # This is not nice, albeit works; have to look for a better solution.
@@ -470,7 +508,7 @@ sub get_mainchain_name
                 $name .= $number;
             }
 
-            if( $chain->length == 1 && $element eq 'S' ) {
+            if( $element eq 'S' && all { element( $_ ) eq 'S' } $chain->vertices ) {
                 $name->append_element( 'sulfa' );
             } else {
                 $name->append_element( $elements{$element}->{prefix} );
@@ -534,6 +572,7 @@ sub get_mainchain_name
     $name = 'toluene'      if $name eq 'methylbenzene';
     $name =~ s/^(\d,\d-)dimethylbenzene$/$1xylene/;
     $name = 'formic acid' if $name eq 'methanoic acid';
+    $name =~ s/methanamide$/formamide/;
     $name =~ s/ethanamide$/acetamide/;
     $name =~ s/ethan(-1-)?o(ate|ic acid)$/acet$2/; # BBv2 P-65.1.1.1
     $name =~ s/benzen(-1-)?amine$/aniline/;
@@ -750,6 +789,19 @@ sub element
     return $atom_or_group->element;
 }
 
+sub charge
+{
+    my( $atom_or_group ) = @_;
+    return undef unless ref $atom_or_group;
+
+    if( !blessed $atom_or_group ) {
+        die "unknown value '$atom_or_group' given for charge()\n" unless ref $atom_or_group eq 'HASH';
+        return exists $atom_or_group->{charge} ? $atom_or_group->{charge} : 0;
+    }
+
+    return $atom_or_group->charge;
+}
+
 # Check if an object or Perl hash is of certain chemical element
 sub is_element
 {
@@ -914,9 +966,9 @@ sub select_mainchain
                                                $copy->degree( $_ ) == 1 &&
                                                element( $_ ) =~ /^(F|Cl|Br|I)$/ }
                                              $copy->vertices );
+                $copy->delete_vertices( map { $_->vertices } $copy->groups );
                 $copy->delete_vertices( grep { !is_element( $_, 'C' ) && !$_->is_part_of_chain }
-                                        grep { blessed $_ }
-                                             $copy->vertices );
+                                        grep { blessed $_ } $copy->vertices );
 
                 my $A = shift @$path;
                 my $B = pop @$path;
@@ -978,6 +1030,7 @@ sub select_mainchain
                                            $_->isa( ChemOnomatopist::Group:: ) &&
                                            !$_->is_part_of_chain }
                                          $subgraph->vertices );
+        die "cannot determine the parent structure\n" unless $subgraph->is_connected;
         my @center = graph_center( $subgraph );
         my @path_parts;
         if( @center == 1 ) {
@@ -998,49 +1051,35 @@ sub select_mainchain
             }
         }
 
-        return shift @path_parts if @path_parts == 1; # methane
-
-        # Generate all possible chains.
-        # FIXME: This needs optimisation.
-        for my $part1 (@path_parts) {
-            for my $part2 (@path_parts) {
-                next if $part1->group eq $part2->group;
-                push @chains, ChemOnomatopist::Chain::FromHalves->new( $part1, $part2 );
+        if( @path_parts > 1 ) {
+            # Generate all possible chains.
+            # FIXME: This needs optimisation.
+            for my $part1 (@path_parts) {
+                for my $part2 (@path_parts) {
+                    next if $part1->group eq $part2->group;
+                    push @chains, ChemOnomatopist::Chain::FromHalves->new( $part1, $part2 );
+                }
             }
+        } else { # methane
+            @chains = @path_parts;
         }
     }
 
     die "cannot determine the parent structure\n" unless @chains;
 
     my $chain = filter_chains( @chains );
+    $chain->{is_main} = 1;
     my @vertices = $chain->vertices;
 
-    # Recognising amide chains
-    if( $most_senior_group && $most_senior_group eq ChemOnomatopist::Group::Amide:: &&
-        @groups == 1 && !$chain->isa( ChemOnomatopist::Chain::Carboxamide:: ) ) {
-        $chain = ChemOnomatopist::Chain::Amide->new( $graph, $chain, @groups );
-    }
-    # Recognising amine chains
-    if( $most_senior_group && $most_senior_group eq ChemOnomatopist::Group::Amine:: &&
-        @groups == 1 && !$chain->isa( ChemOnomatopist::Chain::Bicycle::Purine:: ) ) {
-        $chain = ChemOnomatopist::Chain::Amine->new( $graph, $chain, @groups );
-    }
-    # Recognising imino chains
-    if( $most_senior_group && $most_senior_group eq ChemOnomatopist::Group::Imine:: &&
-        @groups == 1 ) {
-        $chain = ChemOnomatopist::Chain::Imine->new( $graph, $chain, @groups );
+    # Replace the outdated chain with the selected one
+    for my $group ($graph->groups) {
+        next unless set( $group->vertices ) == set( $chain->vertices );
+        $graph->delete_group( $group );
     }
+    $graph->add_group( $chain );
 
-    # This is needed to detect ethers.
-    # However, it clears the cache of chains, thus is quite suboptimal.
-    if( blessed $chain eq ChemOnomatopist::Chain::FromHalves:: ) {
-        $chain = ChemOnomatopist::Chain->new( $graph, undef, @vertices );
-    }
-
-    # Replace the original chain with the selected candidate
-    if( $chain->isa( ChemOnomatopist::Group:: ) && $chain->candidate_for ) {
-        graph_replace( $graph, $chain, $chain->candidate_for );
-    }
+    ChemOnomatopist::Grammar::parse_graph( $graph, @ChemOnomatopist::Grammar::mainchain_rules );
+    $chain = first { $_->is_main } $graph->groups;
 
     # If there is at least one of carbon-based senior group attachment,
     # it means both ends are already senior, prompting to follow the
@@ -1077,6 +1116,10 @@ sub select_sidechain
         # Chalcogen analogues of ethers
         $C_graph->delete_vertices( grep { !is_element( $_, element( $start ) ) }
                                         $C_graph->vertices );
+    } elsif( $parent && grep { element( $start ) && element( $start ) eq $_ } qw( Si ) ) {
+        # Elements which can start sidechains
+        $C_graph->delete_vertices( grep { !is_element( $_, element( $start ) ) }
+                                        $C_graph->vertices );
     } else {
         # Delete non-carbon leaves
         $C_graph->delete_vertices( grep { $_ != $start && !is_element( $_, 'C' ) &&
