@@ -6,23 +6,29 @@ package ChemOnomatopist::Chain::Bicycle::Purine;
 use strict;
 use warnings;
 
+use List::Util qw( first );
+
 use parent ChemOnomatopist::Chain::Bicycle::;
 
 sub new
 {
-    my( $class, $graph, $pyrimidine, $imidazole ) = @_;
+    my( $class, $graph, @cycles ) = @_;
 
+    my $imidazole  = first { $_->length == 5 } @cycles;
+    my $pyrimidine = first { $_->length == 6 } @cycles;
     if( ChemOnomatopist::element( $pyrimidine->{vertices}[0] ) eq 'N' ) {
-        ( $pyrimidine, $imidazole ) = map { $_->flipped }
-                                          ( $pyrimidine, $imidazole );
+        $pyrimidine = $pyrimidine->flipped;
+    }
+    if( $pyrimidine->{vertices}[-1] != $imidazole->{vertices}[-1] ) {
+        $imidazole = $imidazole->flipped;
     }
 
     my @vertices = ( @{$pyrimidine->{vertices}}[1..5],
                      $pyrimidine->{vertices}[0],
-                     reverse @{$imidazole->{vertices}}[0..2] );
+                     @{$imidazole->{vertices}}[0..2] );
     return bless { graph => $graph,
-                   cycles => [ $pyrimidine, $imidazole ],
-                   vertices => \@vertices };
+                   cycles => \@cycles,
+                   vertices => \@vertices }, $class;
 }
 
 sub is_purine() { 1 }
