@@ -7,7 +7,7 @@ use strict;
 use warnings;
 
 use ChemOnomatopist;
-use ChemOnomatopist::Chain; # FIXME: Not sure why it is needed
+use ChemOnomatopist::Chain;
 use ChemOnomatopist::Chain::Monocycle;
 use ChemOnomatopist::Elements qw( %elements );
 use ChemOnomatopist::Name::Part::NondetachablePrefix;
@@ -92,12 +92,13 @@ sub needs_substituent_locants()
 {
     my( $self ) = @_;
     # BBv2 P-14.3.4.2 (c): monosubstituted homogeneous cycles do not need locants
-    return '' if $self->is_homogeneous && $self->number_of_branches == 1;
+    return '' if $self->is_homogeneous && $self->number_of_branches == 1 && !$self->number_of_isotopes;
 
+    # If there is only one kind of substituents, locants are not needed
     if( $self->is_homogeneous && $self->number_of_branches >= $self->max_valence - 1 ) {
-        # If there is only one kind of substituents, locants are not needed
         return '' if uniq( map { "$_" } map { @$_ } $self->locant_names ) == 1;
     }
+
     return 1;
 }
 
