@@ -83,11 +83,15 @@ sub has_form($$)
     return 1;
 }
 
+sub needs_charge_locants() { 1 }
+sub needs_isotope_locants() { 1 }
 sub needs_multiple_bond_locants() { 1 }
 sub needs_substituent_locants() { 1 }
 
 sub locants() { shift; return map { $_ + 1 } @_ }
 
+sub number_of_rings() { 2 }
+
 sub prefix()
 {
     my( $self ) = @_;
