@@ -54,21 +54,40 @@ sub new
 sub candidates()
 {
     my( $self ) = @_;
-    my @candidates = ( $self );
-    my @vertices = $self->vertices;
-
-    push @candidates,
-         bless { graph => $self->graph,
-                 vertices => [ reverse @vertices[11..13], @vertices[0..10] ],
-                 candidate_for => $self };
+    my @candidates = ( $self, $self->flipped_horizontally );
 
     if( $self->number_of_heteroatoms == 2 &&
-        uniq( $self->heteroatoms ) == 1 ) { # TODO: Add two more candidates
+        uniq( $self->heteroatoms ) == 1 ) {
+        push @candidates,
+             $self->flipped_vertically,
+             $self->flipped_vertically->flipped_horizontally;
+    }
+
+    for (1..$#candidates) {
+        $candidates[$_]->{candidate_for} = $self;
     }
 
     return @candidates;
 }
 
+sub flipped_horizontally()
+{
+    my( $self ) = @_;
+    my @vertices = $self->vertices;
+    return bless { graph => $self->graph,
+                   vertices => [ reverse @vertices[11..13], @vertices[0..10] ] };
+}
+
+sub flipped_vertically()
+{
+    my( $self ) = @_;
+    my @vertices = reverse $self->vertices;
+    for (1..4) {
+        unshift @vertices, pop @vertices;
+    }
+    return bless { graph => $self->graph, vertices => \@vertices };
+}
+
 sub locants(@)
 {
     my $self = shift;
@@ -88,6 +107,8 @@ sub needs_heteroatom_locants() { '' }
 sub needs_heteroatom_names() { '' }
 sub needs_substituent_locants() { 1 }
 
+sub number_of_rings() { 3 }
+
 sub prefix()
 {
     my( $self ) = @_;
