@@ -8,11 +8,13 @@ use warnings;
 
 use parent ChemOnomatopist::Group::;
 
+use ChemOnomatopist::Name;
+
 sub element() { 'N' }
 
 sub is_terminal() { 1 }
 
-sub prefix() { 'amino' }
-sub suffix() { 'amine' }
+sub prefix() { ChemOnomatopist::Name->new( 'amino' ) }
+sub suffix() { ChemOnomatopist::Name->new( 'amine' ) }
 
 1;
