@@ -6,9 +6,11 @@ package ChemOnomatopist::Group::Carbonitrile;
 use strict;
 use warnings;
 
+use ChemOnomatopist::Name;
+
 use parent ChemOnomatopist::Group::;
 
-sub prefix { return 'cyano' }
-sub suffix { return 'carbonitrile' }
+sub prefix { ChemOnomatopist::Name->new( 'cyano' ) }
+sub suffix { ChemOnomatopist::Name->new( 'carbonitrile' ) }
 
 1;
