@@ -10,6 +10,7 @@ use parent ChemOnomatopist::Group::;
 
 use ChemOnomatopist::Elements qw( %elements );
 use ChemOnomatopist::Group::Hydroperoxide;
+use ChemOnomatopist::Group::Ketone;
 use ChemOnomatopist::Name;
 use List::Util qw( all any uniq );
 use Scalar::Util qw( blessed );
@@ -17,6 +18,8 @@ use Scalar::Util qw( blessed );
 sub new()
 {
     my( $class, $hydroxy, $ketone ) = @_;
+    die "cannot handle carboxyl group attached to nongroup atoms (most likely hydrazines)\n" unless blessed $ketone;
+    die "cannot handle carboxyl group attached to anything else than ketone\n" unless $ketone->isa( ChemOnomatopist::Group::Ketone:: );
     return bless { hydroxy => $hydroxy, ketone => $ketone }, $class;
 }
 
@@ -71,11 +74,11 @@ sub suffix()
     my $ketone = $self->{ketone};
     my @elements = map { ChemOnomatopist::element( $_ ) } ( $hydroxy, $ketone );
     if( all { $_ eq 'O' } @elements ) {
-        return ChemOnomatopist::Name->new( 'oic acid' ) if blessed $hydroxy;
+        return ChemOnomatopist::Name->new( 'oic acid' ) if blessed $hydroxy && !$hydroxy->charge;
         return ChemOnomatopist::Name->new( 'oate' );
     }
 
-    return element_suffix( @elements ) . 'ate' unless blessed $hydroxy;
+    return element_suffix( @elements ) . 'ate' unless blessed $hydroxy && !$hydroxy->charge;
     return element_suffix( @elements ) . 'ic acid' if uniq( @elements ) == 1;
     return element_suffix( @elements ) . ('ic ' . $elements[0] . '-acid');
 }
@@ -110,12 +113,12 @@ sub multisuffix()
     } else {
         my @elements = ( ChemOnomatopist::element( $hydroxy ), $ketone->element );
         if( all { $_ eq 'O' } @elements ) {
-            return ChemOnomatopist::Name->new( 'carboxylic acid' ) if blessed $hydroxy;
+            return ChemOnomatopist::Name->new( 'carboxylic acid' ) if blessed $hydroxy && !$hydroxy->charge;
             return ChemOnomatopist::Name->new( 'carboxylate' );
         }
 
         $name .= element_suffix( @elements );
-        return $name . 'ate' unless blessed $hydroxy;
+        return $name . 'ate' unless blessed $hydroxy && !$hydroxy->charge;
         return $name . 'ic acid' if uniq( @elements ) == 1;
         return $name . ('ic ' . $elements[0] . '-acid');
     }
