@@ -7,6 +7,7 @@ use strict;
 use warnings;
 
 use ChemOnomatopist::Name;
+use ChemOnomatopist::Name::Part::Isotope;
 
 use parent ChemOnomatopist::Group::;
 
@@ -17,6 +18,7 @@ sub new
 }
 
 sub element() { $_[0]->{atom}{symbol} }
+sub charge()  { ChemOnomatopist::charge( $_[0]->{atom} ) }
 
 # From BBv2 P-63.1.5
 my %prefixes = ( O => 'hydroxy', S => 'sulfanyl', Se => 'selanyl', Te => 'tellanyl' );
@@ -37,11 +39,13 @@ sub suffix
     if( exists $self->{atom}{isotope} ) {
         $suffix = '(' . $self->{atom}{isotope} . $self->element . ')';
     }
-    if( @{$self->{atom}{h_isotope}} && defined $self->{atom}{h_isotope}[0] ) {
+    if( exists $self->{atom}{h_isotope} && @{$self->{atom}{h_isotope}} && defined $self->{atom}{h_isotope}[0] ) {
         $suffix = '(' . $self->{atom}{h_isotope}[0] . 'H)';
     }
 
-    return ChemOnomatopist::Name->new( $suffix . $suffixes{$self->element} );
+    my $name = ChemOnomatopist::Name->new;
+    $name .= ChemOnomatopist::Name::Part::Isotope->new( $suffix ) if $suffix;
+    return $name . $suffixes{$self->element};
 }
 
 sub _cmp_instances
