@@ -1,10 +1,12 @@
 package ChemOnomatopist::Group::Ketone;
 
+# ABSTRACT: Ketone group
+# VERSION
+
 use strict;
 use warnings;
 
-# ABSTRACT: Ketone group
-# VERSION
+use ChemOnomatopist::Name;
 
 use parent ChemOnomatopist::Group::;
 
@@ -12,18 +14,18 @@ use parent ChemOnomatopist::Group::;
 my %prefixes = ( O => 'oxo', S => 'sulfanylidene', Se => 'selanylidene', Te => 'tellanylidene' );
 my %suffixes = ( O => 'one', S => 'thione', Se => 'selone', Te => 'tellone' );
 
-sub needs_multiple_bond_suffix { return '' }
+sub needs_multiple_bond_suffix { '' }
 
 sub prefix
 {
     my( $self ) = @_;
-    return $prefixes{$self->element};
+    return ChemOnomatopist::Name->new( $prefixes{$self->element} );
 }
 
 sub suffix
 {
     my( $self ) = @_;
-    return $suffixes{$self->element};
+    return ChemOnomatopist::Name->new( $suffixes{$self->element} );
 }
 
 sub _cmp_instances
