@@ -1,15 +1,19 @@
 package ChemOnomatopist::Group::Nitro;
 
-use strict;
-use warnings;
-
 # ABSTRACT: Nitro group
 # VERSION
 
+use strict;
+use warnings;
+
 use parent ChemOnomatopist::Group::;
 
-sub prefix { return 'nitro' }
+use ChemOnomatopist::Name;
 
-sub is_prefix_only() { return 1 }
+sub element() { 'N' }
+
+sub is_prefix_only() { 1 }
+
+sub prefix { ChemOnomatopist::Name->new( 'nitro' ) }
 
 1;
