@@ -1,21 +1,23 @@
 package ChemOnomatopist::Group::Sulfinyl;
 
-use strict;
-use warnings;
-
 # ABSTRACT: Sulfinyl group
 # VERSION
 
+use strict;
+use warnings;
+
 use parent ChemOnomatopist::Group::;
 
+use ChemOnomatopist::Name;
+
 my %prefixes = (
     S  => 'sulfinyl',
     Se => 'seleninyl',
     Te => 'tellurinyl',
 );
 
-sub prefix { return $prefixes{$_[0]->element} }
+sub is_prefix_only() { 1 }
 
-sub is_prefix_only() { return 1 }
+sub prefix { ChemOnomatopist::Name->new( $prefixes{$_[0]->element} ) }
 
 1;
