@@ -1,21 +1,23 @@
 package ChemOnomatopist::Group::Sulfonyl;
 
-use strict;
-use warnings;
-
 # ABSTRACT: Sulfonyl group
 # VERSION
 
+use strict;
+use warnings;
+
 use parent ChemOnomatopist::Group::;
 
+use ChemOnomatopist::Name;
+
 my %prefixes = (
     S  => 'sulfonyl',
     Se => 'selenonyl',
     Te => 'telluronyl',
 );
 
-sub prefix { return $prefixes{$_[0]->element} }
+sub prefix() { ChemOnomatopist::Name->new( $prefixes{$_[0]->element} ) }
 
-sub is_prefix_only() { return 1 }
+sub is_prefix_only() { 1 }
 
 1;
