@@ -43,7 +43,7 @@ sub append($)
     my( $self, $name ) = @_;
     return $self if $name eq ''; # Do not do anything if name is empty
 
-    $self->[-1] =~ s/a$// if $name =~ /^a/ && @$self;
+    $self->[-1] =~ s/a$// if $name =~ /^[ai]/ && @$self;
     $self->[-1] =~ s/o$// if $name =~ /^o/ && @$self && $self->[-1] ne 'cyclo';
 
     # If names are combined and the second one starts with a number, a separator is added.
@@ -108,7 +108,8 @@ sub append_suffix($)
 {
     my( $self, $suffix ) = @_;
     if( @$self ) {
-        if( $suffix =~ /^[aeiouy]/ ) {
+        if( ( blessed $suffix && $suffix->remove_isotopes =~ /^[aeiouy]/) ||
+            (!blessed $suffix && $suffix =~ /^[aeiouy]/) ) {
             $self->[-2] =~ s/e$// if $self =~ /e-[0-9,]+-$/; # BBv2 P-16.7.1 (a)
             $self->[-1] =~ s/e$// if $self =~ /e$/;
         }
@@ -208,6 +209,12 @@ sub remove_isotopes()
     return bless { name => \@parts };
 }
 
+sub has_isotope()
+{
+    my( $self ) = @_;
+    return any { blessed $_ && $_->isa( ChemOnomatopist::Name::Part::Isotope:: ) } @$self;
+}
+
 sub has_locant()
 {
     my( $self ) = @_;
